@@ -60,11 +60,6 @@ class GUI:
     SPINBUTTON_DEC_KEY = (Gdk.KEY_minus,Gdk.KEY_less)
     STYLE_ERR = 'dialog_error'
 
-    cursor_date = dt_date.today()
-    cursor_idx_in_date = 0 # cursor index within date
-    today_toggle_date = None
-    today_toggle_idx = 0
-
     views = []
     view_widgets = [] # type: List[Gtk.Widget]
     _view_idx = 0 # take first view as default
@@ -144,6 +139,7 @@ class GUI:
             'menuitem_cut': cls.cut_request,
             'menuitem_copy': cls.copy_request,
             'menuitem_paste': cls.paste_request,
+            'menuitem_find': cls.handler_find,
             'menuitem_newevent': cls.handler_newevent,
             'menuitem_newtodo': cls.handler_newtodo,
             'menuitem_edittime': cls.handler_edittime,
@@ -248,10 +244,7 @@ class GUI:
         cls._init_clipboard()
         cls._init_date_format()
         EventDialogController.init()
-
-        # If date set from command line, jump there now
-        if Config.date:
-            GUI.cursor_date = Config.date
+        FindController.init()
 
         # Wait for calendar to finish initialising before doing views
         while cls._starting_cal:
@@ -276,12 +269,20 @@ class GUI:
         cls._init_views()
         TodoDialogController.init() # Need to do this after Todo View init
 
+        # If view set in config, set index
         vw = Config.get('startup','view')
         if vw:
             for ii in range(len(GUI._VIEWS)):
                 if GUI._VIEWS[ii].lower() == vw:
                     cls._view_idx = ii
                     break
+
+        # If date set from command line, jump there now
+        if Config.date:
+            for i in range(cls._view_idx, cls._view_idx+len(cls.views)):
+                if cls.views[i%len(cls.views)].cursor_set_date(Config.date):
+                    break
+
         cls._box_view_cont.pack_start(cls._eventbox, True, True, 0)
         cls._box_view_cont.reorder_child(cls._eventbox, view_pos)
         cls._eventbox.add(cls.view_widgets[cls._view_idx])
@@ -368,14 +369,14 @@ class GUI:
 
     @classmethod
     def _init_views(cls) -> None:
-        # Get new Gtk Widgets for views.
-        # Add view switching options to menu.
+        # Get Gtk Widgets for views; add view switching options to menu
         for v in GUI._VIEWS:
             m = import_module('.pygenda_view_{:s}'.format(v.lower()),package='pygenda')
             cls.views.append(getattr(m, 'View_{:s}'.format(v)))
             cls.view_widgets.append(cls.views[-1].init())
             cls.view_widgets[-1].get_style_context().add_class('view')
 
+        # Add views to menu, so the user can switch to them
         menu_views_list = cls._builder.get_object('menu_views_list')
         accel_gp = Gtk.AccelGroup()
         cls._window.add_accel_group(accel_gp)
@@ -509,11 +510,11 @@ class GUI:
     def view_redraw(cls, en_changes:bool=False) -> None:
         # Redraw the currently active view.
         # en_changes: bool, True if displayed entries need updating too
-        cls.views[cls._view_idx].redraw(en_changes)
+        cls.views[cls._view_idx].redraw(en_changes=en_changes)
 
 
     @classmethod
-    def switch_view(cls, wid:Gtk.Widget, idx:int=None) -> None:
+    def switch_view(cls, wid:Gtk.Widget, idx:int=None, redraw:bool=True) -> None:
         # Callback from UI widget (e.g. menu, softbutton) to change view.
         # idx = index of new view (otherwise goes to next view in list)
         if idx is None:
@@ -526,7 +527,8 @@ class GUI:
         cls._eventbox.remove(cls._eventbox.get_child())
         new_view = cls.views[cls._view_idx]
         new_view.renew_display()
-        new_view.redraw(True)
+        if redraw:
+            new_view.redraw(en_changes=True)
         new_wid = cls.view_widgets[cls._view_idx]
         cls._eventbox.add(new_wid)
         new_wid.grab_focus()
@@ -534,24 +536,45 @@ class GUI:
 
 
     @classmethod
-    def cursor_set(cls, dt:dt_date, idx:int=None) -> None:
-        # Set current cursor date, and optionally the index within the date.
-        # Call redraw on view if required.
-        if dt != cls.cursor_date or (idx is not None and idx != cls.cursor_idx_in_date):
-            cls.cursor_date = dt
-            if idx is not None:
-                cls.cursor_idx_in_date = idx
-            cls.view_redraw(False)
+    def cursor_goto_date(cls, dt:dt_date) -> None:
+        # Call view to set current cursor date.
+        # If current view does not support setting date (e.g. Todo View)
+        # try next view etc., and make successful view active.
+        # (N.B.: Does not call redraw - caller needs to handle that.)
+        for i in range(cls._view_idx, cls._view_idx+len(cls.views)):
+            v = i%len(cls.views)
+            if cls.views[v].cursor_set_date(dt): # True if view can show date
+                if v != cls._view_idx:
+                    cls.switch_view(None, v, redraw=False)
+                break
 
 
     @classmethod
-    def cursor_inc(cls, delta:timedelta, idx:int=None) -> None:
-        # Add delta to current cursor date; optionally set index in date.
-        # Call redraw on view.
-        cls.cursor_date += delta
-        if idx is not None:
-            cls.cursor_idx_in_date = idx
-        cls.view_redraw(False)
+    def cursor_goto_event(cls, ev:iEvent) -> None:
+        # Call view to move cursor to given event.
+        # If current view does not support showing event (e.g. Todo View),
+        # try next view etc., and make successful view active.
+        # (N.B.: Does not call redraw - caller needs to handle that.)
+        for i in range(cls._view_idx, cls._view_idx+len(cls.views)):
+            v = i%len(cls.views)
+            if cls.views[v].cursor_goto_event(ev): # True if view can show event
+                if v != cls._view_idx:
+                    cls.switch_view(None, v, redraw=False)
+                break
+
+
+    @classmethod
+    def cursor_goto_todo(cls, todo:iTodo, list_idx:int) -> None:
+        # Call view to move cursor to given todo in given todo list.
+        # If current view does not support displaying todo, try
+        # the next view etc., and make successful view active.
+        # (N.B.: Does not call redraw - caller needs to handle that.)
+        for i in range(cls._view_idx, cls._view_idx+len(cls.views)):
+            v = i%len(cls.views)
+            if cls.views[v].cursor_goto_todo(todo, list_idx): # True if view can show todo
+                if v != cls._view_idx:
+                    cls.switch_view(None, v, redraw=False)
+                break
 
 
     # Main
@@ -573,6 +596,11 @@ class GUI:
         date = cls.views[cls._view_idx].cursor_date()
         EventDialogController.newevent(date=date)
 
+    @classmethod
+    def handler_find(cls, *args) -> None:
+        # Callback for find signal (menu)
+        FindController.find()
+
     @classmethod
     def handler_newtodo(cls, *args) -> None:
         # Callback for new todo signal (menu)
@@ -615,7 +643,7 @@ class GUI:
         en = cls.views[cls._view_idx].get_cursor_entry()
         if en:
             Calendar.set_toggle_status_entry(en, stat)
-            cls.view_redraw(True)
+            cls.view_redraw(en_changes=True)
 
 
     @classmethod
@@ -646,7 +674,7 @@ class GUI:
                 calbuf = en.to_ical()
                 cls._lib_clip.set_cb(ctypes.create_string_buffer(txtbuf),ctypes.create_string_buffer(calbuf))
                 Calendar.delete_entry(en)
-                cls.view_redraw(True)
+                cls.view_redraw(en_changes=True)
 
 
     @classmethod
@@ -676,7 +704,7 @@ class GUI:
             ical = iCalendar.from_ical(sdat.get_data())
             en = ical.walk()[0]
             cls.views[cls._view_idx].new_entry_from_example(en)
-            cls.view_redraw(True)
+            cls.view_redraw(en_changes=True)
             return
         except:
             None
@@ -715,7 +743,7 @@ class GUI:
         dialog.destroy()
         if response == Gtk.ResponseType.APPLY:
             Calendar.delete_entry(en)
-            cls.view_redraw(True)
+            cls.view_redraw(en_changes=True)
 
 
     @classmethod
@@ -724,7 +752,7 @@ class GUI:
         dialog = Gtk.Dialog(title=_('Go To'), parent=cls._window,
             flags=Gtk.DialogFlags.MODAL|Gtk.DialogFlags.DESTROY_WITH_PARENT,
             buttons=(Gtk.STOCK_CANCEL, Gtk.ResponseType.CLOSE))
-        wdate = WidgetDate(cls.cursor_date)
+        wdate = WidgetDate(cls.views[cls._view_idx].cursor_date())
         if (not dialog or not wdate): # Allocation check
             raise NameError('Dialog Goto creation failure')
         wdate.connect('changed', GUI.check_date_fixed)
@@ -746,7 +774,8 @@ class GUI:
             # Date is invalid, add error styling
             wdate.get_style_context().add_class(GUI.STYLE_ERR)
         if response == Gtk.ResponseType.APPLY:
-            cls.cursor_set(dt)
+            cls.cursor_goto_date(dt)
+            cls.view_redraw(en_changes=False)
         dialog.destroy()
 
 
@@ -887,7 +916,7 @@ class EventDialogController:
         cls.wid_tabs = GUI._builder.get_object('dialogevent_tabs')
 
         # Create & add event date widget
-        cls.wid_date = WidgetDate(GUI.cursor_date)
+        cls.wid_date = WidgetDate()
         wid_grid.attach(cls.wid_date, 1,1, 1,1)# Can we locate this another way?
         cls.wid_date.show_all()
 
@@ -949,7 +978,7 @@ class EventDialogController:
         cls.wid_rep_forever.connect('toggled', cls._do_repeatforever_toggle)
 
         cls.wid_rep_occs = GUI._builder.get_object('repeat_occurrences')
-        cls.wid_rep_enddt = WidgetDate(GUI.cursor_date)
+        cls.wid_rep_enddt = WidgetDate()
         rbox = GUI._builder.get_object('revealer_repeat_until_e').get_child() # Should be a GtkBox
         rbox.add(cls.wid_rep_enddt)
         rbox.show_all()
@@ -1394,14 +1423,9 @@ class EventDialogController:
         cls._empty_desc_allowed = True # initially allowed, can switch to False
         response,ei = cls._do_event_dialog(txt=txt, date=date)
         if response==Gtk.ResponseType.OK and ei.desc:
-            Calendar.new_entry(ei)
-            if ei.rep_type is None:
-                # Jump to event date (not repeating, so well-defined)
-                # !! This is a quick fix most common case. Need to fix:
-                #    - Multiple entries on one day - jump to correct
-                #    - Repeating entries (jump to visible/closest)
-                GUI.cursor_date = ei.get_start_date()
-            GUI.view_redraw(True)
+            ev = Calendar.new_entry(ei)
+            GUI.cursor_goto_event(ev)
+            GUI.view_redraw(en_changes=True)
 
 
     @classmethod
@@ -1413,10 +1437,8 @@ class EventDialogController:
         if response==Gtk.ResponseType.OK:
             if ei.desc:
                 Calendar.update_entry(event, ei)
-                if ei.rep_type is None:
-                    # Jump to event date
-                    GUI.cursor_date = ei.get_start_date()
-                GUI.view_redraw(True)
+                GUI.cursor_goto_event(event)
+                GUI.view_redraw(en_changes=True)
             else: # Description text has been deleted in dialog
                 GUI.dialog_deleteentry(event)
 
@@ -1889,7 +1911,7 @@ class ExceptionsDialogController:
         self.dialog.get_content_area().add(content)
 
         # Date widget
-        self.wdate = WidgetDate(GUI.cursor_date)
+        self.wdate = WidgetDate()
         self.wdate.connect('changed', GUI.check_date_fixed)
         add_content.add(self.wdate)
 
@@ -2016,27 +2038,29 @@ class TodoDialogController:
     def newtodo(cls, txt:str=None, list_idx:int=None) -> None:
         # Called to implement "new todo" from GUI, e.g. menu
         cls.dialog.set_title(_('New To-do'))
-        response,ei = cls._do_todo_dialog(txt=txt, list_idx=list_idx)
+        response,ei,list_idx = cls._do_todo_dialog(txt=txt, list_idx=list_idx)
         if response==Gtk.ResponseType.OK and ei.desc:
-            Calendar.new_entry(ei)
-            GUI.view_redraw(True)
+            td = Calendar.new_entry(ei)
+            GUI.cursor_goto_todo(td, list_idx)
+            GUI.view_redraw(en_changes=True)
 
 
     @classmethod
     def edittodo(cls, todo:iTodo, list_idx:int=None) -> None:
         # Called to implement "edit todo" from GUI
         cls.dialog.set_title(_('Edit To-do'))
-        response,ei = cls._do_todo_dialog(todo=todo, list_idx=list_idx)
+        response,ei,list_idx = cls._do_todo_dialog(todo=todo, list_idx=list_idx)
         if response==Gtk.ResponseType.OK:
             if ei.desc:
                 Calendar.update_entry(todo, ei)
-                GUI.view_redraw(True)
+                GUI.cursor_goto_todo(todo, list_idx)
+                GUI.view_redraw(en_changes=True)
             else: # Description text has been deleted in dialog
                 GUI.dialog_deleteentry(todo)
 
 
     @classmethod
-    def _do_todo_dialog(cls, txt:str=None, todo:iTodo=None, list_idx:Optional[int]=0) -> Tuple[int,EntryInfo]:
+    def _do_todo_dialog(cls, txt:str=None, todo:iTodo=None, list_idx:Optional[int]=0) -> Tuple[int,EntryInfo,int]:
         # Do the core work displaying todo dialog and extracting result.
         if list_idx==None: # View has not specified a default todo list
             list_idx = 0
@@ -2063,7 +2087,7 @@ class TodoDialogController:
         finally:
             cls.dialog.hide()
 
-        return response,cls._get_entryinfo()
+        return response,cls._get_entryinfo(),cls.wid_todolist.get_active()
 
 
     @classmethod
@@ -2075,3 +2099,62 @@ class TodoDialogController:
         ei.set_categories(cls.list_default_cats[list_idx])
         ei.set_priority(cls.wid_priority.get_active())
         return ei
+
+
+# Singleton class to manage Find dialog
+class FindController:
+    dialog = None # type: Gtk.Dialog
+    find_text = None # type: Gtk.Entry
+
+    @classmethod
+    def init(cls) -> None:
+        # Initialiser for FindController singleton class.
+        # Called from GUI init_stage2().
+
+        # Get some references to dialog elements in glade
+        cls.dialog = GUI._builder.get_object('dialog_find')
+        cls.find_text = GUI._builder.get_object('dialog_find_text')
+        if (not cls.dialog or not cls.find_text): # Sanity check
+            raise NameError('Dialog Find not found')
+
+    @classmethod
+    def find(cls) -> None:
+        # Opens Find dialog and passes search query to _find_results()
+        cls.find_text.grab_focus()
+        response = cls.dialog.run()
+        cls.dialog.hide()
+        if response==Gtk.ResponseType.OK and cls.find_text.get_text():
+            cls._find_results()
+
+
+    @classmethod
+    def _find_results(cls) -> None:
+        # Passes search query to Calendar and displays results in a dialog.
+        r_dialog = Gtk.Dialog(title=_('Find results'), parent=GUI._window,
+            flags=Gtk.DialogFlags.MODAL|Gtk.DialogFlags.DESTROY_WITH_PARENT,
+            buttons=(Gtk.STOCK_CLOSE, Gtk.ResponseType.CLOSE))
+        r_dialog.set_icon_name('edit-find')
+        txt = cls.find_text.get_text()
+        res = Calendar.search(txt)
+        if not res:
+            r_dialog.get_content_area().add(Gtk.Label(_('No results found')))
+        else:
+            scroller = Gtk.ScrolledWindow()
+            scroller.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
+            scroller.set_overlay_scrolling(False)
+            r_grid = Gtk.Grid()
+            scroller.get_style_context().add_class('find_results')
+            scroller.add(r_grid)
+            r_dialog.get_content_area().add(scroller)
+            i = 0
+            for en in res:
+                dt = en['DTSTART'].dt
+                l_dt = Gtk.Label(dt.strftime(GUI.date_formatting_numeric))
+                r_grid.attach(l_dt, 0, i, 1, 1)
+                l_sum = Gtk.Label(en['SUMMARY'])
+                l_sum.set_halign(Gtk.Align.START)
+                r_grid.attach(l_sum, 1, i, 1, 1)
+                i += 1
+        r_dialog.show_all()
+        r_dialog.run()
+        r_dialog.destroy()
