@@ -48,6 +48,9 @@ class View_Todo(View):
     _last_cursor_list = None
     _last_cursor_idx_in_list = None
     _list_items = None # type: list
+    _target_listidx = None
+    _target_todo = None
+    _scroll_to_cursor_required = False
     CURSOR_STYLE = 'todoview_cursor'
 
     @staticmethod
@@ -119,6 +122,7 @@ class View_Todo(View):
         list_hbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
         list_hbox.set_homogeneous(True)
         cls._topboxscroll.add(list_hbox)
+        list_hbox.connect('draw', cls._pre_draw)
 
         # Now add vertical boxes for each list
         cls._list_container = []
@@ -126,13 +130,20 @@ class View_Todo(View):
             new_list = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
             new_list.get_style_context().add_class('todoview_list')
             new_title = Gtk.Label(cls._list_titles[i])
-            new_list.add(new_title)
             new_title.get_style_context().add_class('todoview_title')
+            new_eventbox_title = Gtk.EventBox()
+            new_eventbox_title.connect('button_press_event', cls.click_title, i)
+            new_eventbox_title.add(new_title)
+            new_list.add(new_eventbox_title)
             new_list_scroller = Gtk.ScrolledWindow()
             new_list_scroller.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
             new_list_scroller.set_overlay_scrolling(False)
             new_list_scroller.set_vexpand(True)
-            new_list.add(new_list_scroller)
+            new_list_scroller.get_style_context().add_class('todoview_listcontent')
+            new_eventbox_list = Gtk.EventBox()
+            new_eventbox_list.connect('button_press_event', cls.click_list, i)
+            new_eventbox_list.add(new_list_scroller)
+            new_list.add(new_eventbox_list)
             new_list_content = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
             new_list_scroller.add(new_list_content)
             cls._list_container.append(new_list_scroller)
@@ -170,7 +181,8 @@ class View_Todo(View):
         # Creates new entry based on entry en. Used for pasting entries.
         # Type of entry depends on View (e.g. Todo View -> to-do item).
         cats = cls._list_default_cats[cls._cursor_list]
-        Calendar.new_entry_from_example(en, e_type=EntryInfo.TYPE_TODO, e_cats=cats)
+        new_en = Calendar.new_entry_from_example(en, e_type=EntryInfo.TYPE_TODO, e_cats=cats)
+        cls.cursor_goto_todo(new_en, cls._cursor_list)
 
 
     @classmethod
@@ -188,6 +200,17 @@ class View_Todo(View):
         return cls._cursor_list
 
 
+    @classmethod
+    def cursor_goto_todo(cls, todo:iTodo, list_idx:int) -> bool:
+        # Move cursor to given todo in given list.
+        # Return True to indicate this is possible in this view.
+        # Set targets, but don't move yet, since we want to redraw view
+        # & to use recalculated todo order to determine cursor position.
+        cls._target_listidx = list_idx
+        cls._target_todo = todo
+        return True
+
+
     @classmethod
     def redraw(cls, en_changes:bool) -> None:
         # Called when redraw required
@@ -206,6 +229,13 @@ class View_Todo(View):
             cls._list_items.append([])
             for td in todos:
                 if cls._todo_matches_filter(td, cls._list_filters[i]):
+                    if cls._target_todo is not None and cls._target_todo is td:
+                        cls._cursor_list = i
+                        cls._cursor_idx_in_list = count
+                        if i>=cls._target_listidx:
+                            # We've reached the target list, so go no further
+                            cls._target_listidx = None
+                            cls._target_todo = None
                     row = Gtk.Box()
                     # Potential markers: ①-0x245f ➀-0x277f ❶-0x2775 ➊-0x2789
                     mark_label = Gtk.Label(chr(0x2789+td['PRIORITY']) if 'PRIORITY' in td else u'•')
@@ -234,6 +264,9 @@ class View_Todo(View):
             new_list_content.get_style_context().add_class('todoview_items')
             cls._list_container[i].add(new_list_content)
             cls._list_container[i].show_all()
+        # Reset target (though in theory this should already be done)
+        cls._target_listidx = None
+        cls._target_todo = None
         cls._show_cursor()
 
 
@@ -267,6 +300,54 @@ class View_Todo(View):
         return cats
 
 
+    @classmethod
+    def click_title(cls, wid:Gtk.Widget, ev:Gdk.EventButton, list_idx:int) -> None:
+        # Callback. Called whenever a list title is clicked/tapped.
+        # Moves cursor to top of list/item clicked
+        cls._cursor_move_list(list_idx)
+        cls._cursor_move_index(0)
+        cls._scroll_to_cursor_required = True
+
+
+    @classmethod
+    def click_list(cls, wid:Gtk.Widget, ev:Gdk.EventButton, list_idx:int) -> None:
+        # Callback. Called whenever list content is clicked/tapped.
+        # Moves cursor to list/item clicked
+        cls._cursor_move_list(list_idx)
+        scroller = cls._list_container[list_idx]
+        vwport = scroller.get_children()[0]
+        row_vbox = vwport.get_children()[0]
+        # Pixels above the first row:
+        ycount = cls._top_spacing(scroller)
+        ycount += cls._top_spacing(vwport)
+        ycount += cls._top_spacing(row_vbox)
+        # Take account of list's vertical scrollbar:
+        ycount -= cls._list_container[list_idx].get_vadjustment().get_value()
+        # Look through rows until cumulative height exceeds click ycoord
+        row_spacing = row_vbox.get_spacing()
+        rows = row_vbox.get_children()
+        ycount -= row_spacing//2 # adjust for no spacing above first row
+        i = 0
+        for r in rows:
+            ycount += r.get_allocated_height()
+            ycount += row_spacing
+            if ycount > ev.y:
+                break
+            i += 1
+        cls._cursor_move_index(i)
+        cls._scroll_to_cursor_required = True
+
+
+    @staticmethod
+    def _top_spacing(wid:Gtk.Widget) -> float:
+        # Return total size of top padding+border+margin of widget
+        ctx = wid.get_style_context()
+        pad = ctx.get_padding(Gtk.StateFlags.NORMAL)
+        bord = ctx.get_border(Gtk.StateFlags.NORMAL)
+        marg = ctx.get_margin(Gtk.StateFlags.NORMAL)
+        return pad.top + bord.top + marg.top
+
+
     @classmethod
     def _show_cursor(cls) -> None:
         # Locates bullet corresponding to the current cursor and adds
@@ -286,7 +367,19 @@ class View_Todo(View):
             ctx.add_class(cls.CURSOR_STYLE)
         cls._last_cursor_list = cls._cursor_list
         cls._last_cursor_idx_in_list = cls._cursor_idx_in_list
-        cls._scroll_to_cursor()
+        # Add scroll call to idle, in case column widths not yet determined
+        cls._scroll_to_cursor_required = True
+
+
+    @classmethod
+    def _pre_draw(cls, wid:Gtk.Widget, _) -> None:
+        # Callback called on 'draw' event on date_content.
+        # Called before drawing date content.
+        # Used to scroll window when cursor has been moved (since we
+        # need to have calculated the layout to know where to scoll to).
+        if cls._scroll_to_cursor_required:
+            cls._scroll_to_cursor()
+            cls._scroll_to_cursor_required = False
 
 
     @classmethod
@@ -317,8 +410,8 @@ class View_Todo(View):
     @classmethod
     def _scroll_to_cursor(cls) -> None:
         # If required, scroll view elements so that cursor is visible.
-        # Note: On first call view is not yet drawn, so calculation is not
-        # correct. However, since don't need to scroll, behaviour is right.
+        # Note: If view is not yet laid-out, calculation not correct.
+        # Hence this will be called in 'draw' event handler, so layout is done.
         list_width = cls._list_container[0].get_allocated_width() # homogeneous
         view_width = cls._topboxscroll.get_allocated_width()
         maxv = cls._cursor_list*list_width # left edge of list at left of view
@@ -359,7 +452,7 @@ class View_Todo(View):
 
     @classmethod
     def keypress(cls, wid:Gtk.Widget, ev:Gdk.EventKey) -> None:
-        # Handle key press event in Week view.
+        # Handle key press event in Todo view.
         # Called (from GUI.keypress()) on keypress (or repeat) event
         try:
             f = cls._KEYMAP[ev.keyval]
