@@ -25,13 +25,13 @@ gi_require_version('Gtk', '3.0')
 from gi.repository import Gtk, Gdk, GLib
 
 from calendar import day_abbr,month_name
-from datetime import time as dt_time, date as dt_date, timedelta
+from datetime import time as dt_time, date as dt_date, datetime as dt_datetime, timedelta
 from icalendar import cal as iCal
 from locale import gettext as _
 from typing import Optional
 
 # pygenda components
-from .pygenda_view import View_DayUnit_Base
+from .pygenda_view import View, View_DayUnit_Base
 from .pygenda_calendar import Calendar
 from .pygenda_config import Config
 from .pygenda_util import start_of_week, day_in_week, month_abbr, start_end_dts_occ, dt_lte
@@ -46,10 +46,11 @@ class View_Week(View_DayUnit_Base):
     })
 
     _day_ent_count = [0]*7 # entry count for each day
-    _visible_occurrences = []
+    _day_entries = [[], [], [], [], [], [], []]
     _week_viewed = None # So view will be fully redrawn when needed
     _last_cursor = None
-    _scroll_callback_id = None
+    _scroll_to_cursor_in_day = None
+    _target_entry = None
     CURSOR_STYLE = 'weekview_cursor'
 
     @staticmethod
@@ -125,6 +126,7 @@ class View_Week(View_DayUnit_Base):
             day_scroller.add(cls._day_rows[i])
             ctx = cls._day_rows[i].get_style_context()
             ctx.add_class('weekview_daytext')
+            cls._day_rows[i].connect('draw', cls._pre_datecontent_draw, i)
             day_box.pack_start(day_scroller, True, True, 0)
 
         # Attach elements to pages
@@ -141,7 +143,7 @@ class View_Week(View_DayUnit_Base):
         # Sets date and month label text and style classes.
         # Called on view redraw.
         cls._last_cursor = None
-        dt = start_of_week(GUI.cursor_date)
+        dt = start_of_week(View._cursor_date)
         cls._week_viewed = dt
         dt_end = dt + timedelta(days=6)
         # Month label
@@ -191,16 +193,18 @@ class View_Week(View_DayUnit_Base):
         # Sets label text and style classes for event-displaying labels.
         # Called on view redraw.
         cls._last_cursor = None
-        dt = start_of_week(GUI.cursor_date)
-        cls._visible_occurrences = Calendar.occurrence_list(dt, dt+timedelta(days=7))
-        itr = iter(cls._visible_occurrences)
+        dt = start_of_week(View._cursor_date)
+        cls._day_entries = [[], [], [], [], [], [], []] # reset stored events
+        cls._day_ent_count = [0]*7
+        sorted_occurrences = Calendar.occurrence_list(dt, dt+timedelta(days=7))
+        itr = iter(sorted_occurrences)
         try:
             occ = next(itr)
         except StopIteration:
             occ = None
-        cls._day_ent_count = [0]*7 # we'll fill this in as we display
+        oneday = timedelta(days=1)
         for i in range(7):
-            dt_nxt = dt + timedelta(days=1)
+            dt_nxt = dt + oneday
             # Delete anything previously written to day v-box
             cls._day_rows[i].foreach(Gtk.Widget.destroy)
             while True:
@@ -210,19 +214,11 @@ class View_Week(View_DayUnit_Base):
                 if dt_lte(dt_nxt, occ_dt_sta):
                     # into next day so break this loop
                     break
-                row = Gtk.Box()
-                # Create entry mark (bullet or time) & add to row
-                mark_label = cls.marker_label(occ[0], occ_dt_sta)
-                ctx = mark_label.get_style_context()
-                ctx.add_class('weekview_marker') # add style for CSS
-                row.add(mark_label)
-
-                # Create entry content label & add to row
-                cont_label = cls.entry_text_label(occ[0],occ_dt_sta,occ_dt_end)
-                cont_label.set_hexpand(True) # Also sets hexpand_set to True
-                row.add(cont_label)
-                cls._day_rows[i].add(row)
-                cls._day_ent_count[i] += 1
+                # First, see if we've hit the cursor target entry
+                if cls._target_entry is not None and cls._target_entry is occ[0] and dt==View._cursor_date:
+                    View._cursor_idx_in_date = cls._day_ent_count[i]
+                    cls._target_entry = None
+                cls._add_day_entry_row(occ[0], occ_dt_sta, occ_dt_end, i)
                 try:
                     occ = next(itr)
                 except StopIteration:
@@ -236,18 +232,39 @@ class View_Week(View_DayUnit_Base):
                 cls._day_rows[i].add(mark_label)
             dt = dt_nxt
             cls._day_rows[i].show_all()
+        cls._target_entry = None # just in case - should be done already
+
+
+    @classmethod
+    def _add_day_entry_row(cls, ev:iCal.Event, dt_st:dt_date, dt_end:dt_date, dayidx:int) -> None:
+        # Add Gtk labels for event 'ev', occurrence at time/date from 'dt_st'
+        # to 'dt_end', in day 'dayidx' (e.g. 0=Monday if week starts Monday).
+        # Used when displaying week contents.
+        row = Gtk.Box()
+        # Create entry mark (bullet or time) & add to row
+        mark_label = cls.marker_label(ev, dt_st)
+        ctx = mark_label.get_style_context()
+        ctx.add_class('weekview_marker') # add style for CSS
+        row.add(mark_label)
+        # Create entry content label & add to row
+        cont_label = cls.entry_text_label(ev, dt_st, dt_end)
+        cont_label.set_hexpand(True) # Also sets hexpand_set to True
+        row.add(cont_label)
+        cls._day_rows[dayidx].add(row)
+        cls._day_entries[dayidx].append(ev)
+        cls._day_ent_count[dayidx] += 1
 
 
     @classmethod
     def _show_cursor(cls) -> None:
         # Locates bullet/date corresponding to the current cursor and adds
         # 'weekview_cursor' class to it so cursor is visible via CSS styling.
-        dy = day_in_week(GUI.cursor_date)
+        dy = day_in_week(View._cursor_date)
         ecount = cls._day_ent_count[dy]
-        i = GUI.cursor_idx_in_date
+        i = View._cursor_idx_in_date
         if i < 0 or i >= ecount:
             i = max(0,ecount-1)
-            GUI.cursor_idx_in_date = i
+            View._cursor_idx_in_date = i
         cls._hide_cursor()
         row = cls._day_rows[dy].get_children()[i]
         if ecount==0:
@@ -257,25 +274,23 @@ class View_Week(View_DayUnit_Base):
         ctx = mk.get_style_context()
         ctx.add_class(cls.CURSOR_STYLE)
         cls._last_cursor = int(dy+8*i)
-        if cls._scroll_callback_id is not None:
-            # Cancel existing callback to prevent inconsistent scroll requests.
-            GLib.source_remove(cls._scroll_callback_id)
-            cls._scroll_callback_id = None
         if cls._day_ent_count[dy] > 0:
             # We may need to scroll content to show entry at cursor.
-            # Want this to happen after redraw, otherwise, can't access
-            # dimensions of elements needed to calculate scroll size (because
-            # they haven't been calculated). So delay scroll to after redraw.
-            # We save the returned id so the callback can be cancelled.
-            cls._scroll_callback_id = GLib.idle_add(cls._scroll_to_row, cls._day_rows[dy], i, cls._day_scroll[dy], priority=GLib.PRIORITY_HIGH_IDLE+30)
+            cls._day_rows[dy].queue_draw()
+            cls._scroll_to_cursor_in_day = dy # to be read in draw handler
+        else:
+            cls._scroll_to_cursor_in_day = None
 
 
     @classmethod
-    def _scroll_to_row(cls, rowbox:Gtk.Box, row:int, scroller:Gtk.ScrolledWindow) -> bool:
-        # Local version of scroll_to_row() that clears the id, then
-        # calls the parent class scroll_to_row() to do the work.
-        cls._scroll_callback_id = None
-        return cls.scroll_to_row(rowbox, row, scroller)
+    def _pre_datecontent_draw(cls, wid:Gtk.Widget, _, day:int) -> None:
+        # Callback called on 'draw' event on date_content.
+        # Called before drawing date content.
+        # Used to scroll window when cursor has been moved (since we
+        # need to have calculated the layout to know where to scoll to).
+        if cls._scroll_to_cursor_in_day == day:
+            cls.scroll_to_row(cls._day_rows[day], View._cursor_idx_in_date, cls._day_scroll[day])
+            cls._scroll_to_cursor_in_day = None
 
 
     @classmethod
@@ -296,16 +311,23 @@ class View_Week(View_DayUnit_Base):
             cls._last_cursor = None
 
 
+    @classmethod
+    def cursor_set_date(cls, dt:dt_date, idx:int=0) -> bool:
+        # Set current cursor date & index in date.
+        # Override default method & return True to indicate success.
+        View._cursor_date = dt
+        View._cursor_idx_in_date = idx
+        return True
+
+
     @classmethod
     def get_cursor_entry(cls) -> iCal.Event:
         # Returns entry at cursor position, or None if cursor not on entry.
         # Called from cursor_edit_entry() & delete_request().
-        dy = day_in_week(GUI.cursor_date)
+        dy = day_in_week(View._cursor_date)
         if cls._day_ent_count[dy]==0:
             return None
-        i = sum(cls._day_ent_count[:dy])
-        i += GUI.cursor_idx_in_date
-        return cls._visible_occurrences[i][0]
+        return cls._day_entries[dy][View._cursor_idx_in_date]
 
 
     @classmethod
@@ -318,7 +340,7 @@ class View_Week(View_DayUnit_Base):
     def redraw(cls, en_changes:bool) -> None:
         # Called when redraw required.
         # en_changes: bool indicating if displayed entries need updating too
-        if cls._week_viewed != start_of_week(GUI.cursor_date):
+        if cls._week_viewed != start_of_week(View._cursor_date):
             cls._set_label_text()
             en_changes = True
         if en_changes:
@@ -329,11 +351,11 @@ class View_Week(View_DayUnit_Base):
     @classmethod
     def _cursor_move_up(cls) -> None:
         # Callback for user moving cursor up.
-        GUI.cursor_idx_in_date -= 1
-        if GUI.cursor_idx_in_date < 0:
-            GUI.cursor_inc(timedelta(days=-1))
+        View._cursor_idx_in_date -= 1
+        if View._cursor_idx_in_date < 0:
+            cls.cursor_inc(timedelta(days=-1))
             # Leave idx_in_date as -1 since that signals last entry
-            GUI.today_toggle_date = None
+            View._today_toggle_date = None
         else:
             cls._show_cursor()
 
@@ -341,11 +363,11 @@ class View_Week(View_DayUnit_Base):
     @classmethod
     def _cursor_move_dn(cls) -> None:
         # Callback for user moving cursor down.
-        GUI.cursor_idx_in_date += 1
-        dy = day_in_week(GUI.cursor_date)
-        if GUI.cursor_idx_in_date >= cls._day_ent_count[dy]:
-            GUI.cursor_inc(timedelta(days=1), 0)
-            GUI.today_toggle_date = None
+        View._cursor_idx_in_date += 1
+        dy = day_in_week(View._cursor_date)
+        if View._cursor_idx_in_date >= cls._day_ent_count[dy]:
+            cls.cursor_inc(timedelta(days=1), 0)
+            View._today_toggle_date = None
         else:
             cls._show_cursor()
 
@@ -353,31 +375,33 @@ class View_Week(View_DayUnit_Base):
     @classmethod
     def _cursor_move_lt(cls) -> None:
         # Callback for user moving cursor left.
-        i = day_in_week(GUI.cursor_date)
+        i = day_in_week(View._cursor_date)
         d = -7 if i<3 else (-3 if i==3 else -4)
-        GUI.cursor_inc(timedelta(days=d), 0)
-        GUI.today_toggle_date = None
+        cls.cursor_inc(timedelta(days=d), 0)
+        View._today_toggle_date = None
 
 
     @classmethod
     def _cursor_move_rt(cls) -> None:
         # Callback for user moving cursor right.
-        i = day_in_week(GUI.cursor_date)
+        i = day_in_week(View._cursor_date)
         d = 4 if i<3 else 7
-        GUI.cursor_inc(timedelta(days=d), 0)
-        GUI.today_toggle_date = None
+        cls.cursor_inc(timedelta(days=d), 0)
+        View._today_toggle_date = None
 
 
     @classmethod
     def _cursor_move_today(cls) -> None:
         # Callback for user toggling cursor between today & another date
         today = dt_date.today()
-        if GUI.cursor_date != today:
-            GUI.today_toggle_date = GUI.cursor_date
-            GUI.today_toggle_idx = GUI.cursor_idx_in_date
-            GUI.cursor_set(today,0)
-        elif GUI.today_toggle_date is not None:
-            GUI.cursor_set(GUI.today_toggle_date,GUI.today_toggle_idx)
+        if View._cursor_date != today:
+            View._today_toggle_date = View._cursor_date
+            View._today_toggle_idx = View._cursor_idx_in_date
+            cls.cursor_set_date(today)
+            cls.redraw(en_changes=False)
+        elif View._today_toggle_date is not None:
+            cls.cursor_set_date(View._today_toggle_date, idx=View._today_toggle_idx)
+            cls.redraw(en_changes=False)
 
 
     @classmethod
@@ -430,7 +454,14 @@ class View_Week(View_DayUnit_Base):
                 # We're not clicking in date label area, use y to calc entry
                 new_idx = cls.y_to_day_row(cls._day_rows[new_day], ev.y, cls._day_ent_count[new_day], cls._day_scroll[new_day])
 
-        GLib.idle_add(GUI.cursor_set, cls._day_index_to_date(new_day), new_idx)
+        GLib.idle_add(cls._jump_to_date, cls._day_index_to_date(new_day), new_idx)
+
+
+    @classmethod
+    def _jump_to_date(cls, dt:dt_date, idx:int) -> None:
+        # Idle callback to move cursor to given date, e.g. on click
+        cls.cursor_set_date(dt, idx=idx)
+        cls.redraw(en_changes=False)
 
 
     @classmethod
