@@ -31,7 +31,7 @@ from icalendar import cal as iCal
 from typing import Tuple
 
 # pygenda components
-from .pygenda_view import View_DayUnit_Base
+from .pygenda_view import View, View_DayUnit_Base
 from .pygenda_gui import GUI, EventDialogController
 from .pygenda_config import Config
 from .pygenda_calendar import Calendar
@@ -47,11 +47,13 @@ class View_Year(View_DayUnit_Base):
     GRID_CURSOR_STYLE = 'yearview_cursor'
     ENTRY_CURSOR_STYLE = 'yearview_entry_cursor'
     _target_col = None
-    _year_viewed = None
+    _year_viewed = -1 # Indicates next redraw will draw year
     _last_cursor = None
     _visible_occurrences = None
     _show_datecontent_pending = False
     _date_content_count = 0
+    _scroll_to_cursor_required = False
+    _target_entry = None
 
     @staticmethod
     def view_name() -> str:
@@ -75,8 +77,8 @@ class View_Year(View_DayUnit_Base):
         cls._date_label = GUI._builder.get_object('year_datelabel')
         cls._date_content_scroll = GUI._builder.get_object('year_datecontent_scroll')
         cls._date_content = GUI._builder.get_object('year_datecontent')
+        cls._date_content.connect('draw', cls._pre_datecontent_draw)
         cls._draw_day_month_labels()
-        cls._year_viewed = -1 # Indicates next redraw will draw year
         cls._init_keymap()
         cls._init_grid()
         GUI._builder.get_object('year_grid_events').connect('button_press_event', cls.click_grid)
@@ -125,6 +127,8 @@ class View_Year(View_DayUnit_Base):
         cls._KEYMAP_SHIFT = {
             Gdk.KEY_Up: lambda: cls._idxcursor_move_up(),
             Gdk.KEY_Down: lambda: cls._idxcursor_move_dn(),
+            Gdk.KEY_Page_Up: lambda: cls._idxcursor_move_up(), # !! Placeholder
+            Gdk.KEY_Page_Down: lambda: cls._idxcursor_move_dn(),#!! Placeholder
         }
 
 
@@ -144,7 +148,7 @@ class View_Year(View_DayUnit_Base):
         # Draws year - adds formatting classes to cells, labels to some dates.
         # Called on redraw if year has changed.
         # Function a bit disorganised, tidying might be beneficial...
-        c_date = GUI.cursor_date
+        c_date = View._cursor_date
         yr = c_date.year
         cls._year_viewed = yr
         l = GUI._builder.get_object('year_yearlabel')
@@ -235,7 +239,7 @@ class View_Year(View_DayUnit_Base):
     def redraw(cls, en_changes:bool) -> None:
         # Called when redraw required.
         # en_changes: bool indicating if displayed entries need updating too
-        if cls._year_viewed != GUI.cursor_date.year:
+        if cls._year_viewed != View._cursor_date.year:
             cls._draw_year()
             cls._last_cursor = None
             en_changes = True
@@ -257,7 +261,7 @@ class View_Year(View_DayUnit_Base):
     def _show_datelabel(cls) -> None:
         # Show cursor date in label in bottom panel of view.
         # Called on redraw
-        st = GUI.cursor_date.strftime(GUI.date_formatting_text_noyear)+':'
+        st = View._cursor_date.strftime(GUI.date_formatting_text_noyear)+':'
         cls._date_label.set_text(st)
 
 
@@ -266,7 +270,7 @@ class View_Year(View_DayUnit_Base):
         # Show events for current cursor date in bottom panel of view.
         # Assumes that no events are currently shown.
         # Can be slow, so called in idle from redraw.
-        dt = GUI.cursor_date
+        dt = View._cursor_date
         cls._visible_occurrences = Calendar.occurrence_list(dt, dt+timedelta(days=1))
         r = 0
         for occ in cls._visible_occurrences:
@@ -282,11 +286,16 @@ class View_Year(View_DayUnit_Base):
             cont_label.set_hexpand(True) # Also sets hexpand_set to True
             row.add(cont_label)
             cls._date_content.add(row)
+            # See if we've hit the cursor target entry
+            if cls._target_entry is not None and cls._target_entry is occ[0]:
+                View._cursor_idx_in_date = r
+                cls._target_entry = None
             r += 1
         cls._date_content_count = r
         cls._date_content.show_all()
         cls._show_entry_cursor()
         cls._show_datecontent_pending = False
+        cls._target_entry = None # just in case - should be done already
 
 
     @classmethod
@@ -294,17 +303,28 @@ class View_Year(View_DayUnit_Base):
         # Set style to display entry cursor (= cursor in lower section of view)
         if cls._date_content_count==0:
             # No entries for day - return
-            GUI.cursor_idx_in_date = 0
+            View._cursor_idx_in_date = 0
             return
-        i = GUI.cursor_idx_in_date
+        i = View._cursor_idx_in_date
         if i<0 or i>=cls._date_content_count:
             i = cls._date_content_count-1
-            GUI.cursor_idx_in_date = i
+            View._cursor_idx_in_date = i
         mk = cls._date_content.get_children()[i].get_children()[0]
         ctx = mk.get_style_context()
         ctx.add_class(cls.ENTRY_CURSOR_STYLE)
         cls._last_entry_cursor = i
-        cls.scroll_to_row(cls._date_content, i, cls._date_content_scroll)
+        cls._scroll_to_cursor_required = True # to be read in draw handler
+
+
+    @classmethod
+    def _pre_datecontent_draw(cls, wid:Gtk.Widget, _) -> None:
+        # Callback called on 'draw' event on date_content.
+        # Called before drawing date content.
+        # Used to scroll window when cursor has been moved (since we
+        # need to have calculated the layout to know where to scoll to).
+        if cls._scroll_to_cursor_required:
+            cls.scroll_to_row(cls._date_content, View._cursor_idx_in_date, cls._date_content_scroll)
+            cls._scroll_to_cursor_required = False
 
 
     @classmethod
@@ -317,13 +337,25 @@ class View_Year(View_DayUnit_Base):
             cls._last_entry_cursor = None
 
 
+    @classmethod
+    def cursor_set_date(cls, dt:dt_date, idx:int=0, reset_target_col:bool=True) -> bool:
+        # Set current cursor date & index in date.
+        # Override default method so we can update state of target column.
+        # Return True to indicate view supports jumping to a date.
+        View._cursor_date = dt
+        View._cursor_idx_in_date = idx
+        if reset_target_col:
+            cls._target_col = None
+        return True
+
+
     @classmethod
     def get_cursor_entry(cls) -> iCal.Event:
         # Returns entry at cursor position, or None if cursor not on entry.
         # Called from cursor_edit_entry() & delete_request().
         if cls._date_content_count == 0:
             return None
-        return cls._visible_occurrences[GUI.cursor_idx_in_date][0]
+        return cls._visible_occurrences[View._cursor_idx_in_date][0]
 
 
     @classmethod
@@ -388,7 +420,7 @@ class View_Year(View_DayUnit_Base):
     @classmethod
     def _show_cursor(cls) -> None:
         # Add style class for grid cursor to the appropriate cell label
-        new_coords = cls._date_to_cell(GUI.cursor_date)
+        new_coords = cls._date_to_cell(View._cursor_date)
         if cls._last_cursor!=new_coords:
             cls._hide_cursor()
         l = cls._grid_cells.get_child_at(*new_coords)
@@ -410,15 +442,15 @@ class View_Year(View_DayUnit_Base):
     @classmethod
     def _cursor_move_lr(cls, d:int) -> None:
         # Callback for user moving grid cursor left (d<0) or right (d>0).
-        GUI.cursor_inc(timedelta(days=d), 0)
+        cls.cursor_inc(timedelta(days=d), 0)
         cls._target_col = None
-        GUI.today_toggle_date = None
+        View._today_toggle_date = None
 
 
     @classmethod
     def _cursor_move_up(cls) -> None:
         # Callback for user moving grid cursor up.
-        cur_dt = GUI.cursor_date
+        cur_dt = View._cursor_date
         if cls._target_col is None:
             cls._target_col = cls._date_to_cell(cur_dt)[0]
         new_y = cur_dt.month-2 # months 1..12, cells 0..11
@@ -427,14 +459,15 @@ class View_Year(View_DayUnit_Base):
             new_y = 11
             new_yr -= 1
         new_dt = cls._cell_to_date_clamped(cls._target_col, new_y, new_yr)
-        GUI.cursor_set(new_dt,0)
-        GUI.today_toggle_date = None
+        cls.cursor_set_date(new_dt, reset_target_col=False)
+        cls.redraw(en_changes=False)
+        View._today_toggle_date = None
 
 
     @classmethod
     def _cursor_move_dn(cls) -> None:
         # Callback for user moving grid cursor down.
-        cur_dt = GUI.cursor_date
+        cur_dt = View._cursor_date
         if cls._target_col is None:
             cls._target_col = cls._date_to_cell(cur_dt)[0]
         new_y = cur_dt.month # months 1..12, cells 0..11
@@ -443,61 +476,63 @@ class View_Year(View_DayUnit_Base):
             new_y = 0
             new_yr += 1
         new_dt = cls._cell_to_date_clamped(cls._target_col, new_y, new_yr)
-        GUI.cursor_set(new_dt,0)
-        GUI.today_toggle_date = None
+        cls.cursor_set_date(new_dt, reset_target_col=False)
+        cls.redraw(en_changes=False)
+        View._today_toggle_date = None
 
 
     @classmethod
     def _cursor_move_stmon(cls) -> None:
         # Callback for user moving grid cursor to start of month.
-        new_dt = GUI.cursor_date.replace(day=1)
-        GUI.cursor_set(new_dt,0)
-        cls._target_col = None
-        GUI.today_toggle_date = None
+        new_dt = View._cursor_date.replace(day=1)
+        cls.cursor_set_date(new_dt, reset_target_col=True)
+        cls.redraw(en_changes=False)
+        View._today_toggle_date = None
 
 
     @classmethod
     def _cursor_move_endmon(cls) -> None:
         # Callback for user moving grid cursor to end of month.
-        dt = GUI.cursor_date
+        dt = View._cursor_date
         lastday = calendar.monthrange(dt.year,dt.month)[1]
         new_dt = dt.replace(day=lastday)
-        GUI.cursor_set(new_dt,0)
-        cls._target_col = None
-        GUI.today_toggle_date = None
+        cls.cursor_set_date(new_dt, reset_target_col=True)
+        cls.redraw(en_changes=False)
+        View._today_toggle_date = None
 
 
     @classmethod
     def _cursor_move_pgupdn(cls, d:int) -> None:
         # Callback for user moving grid cursor pageup (d<0) or pagedown (d>0).
-        cur_dt = GUI.cursor_date
+        cur_dt = View._cursor_date
         if cls._target_col is None:
             cls._target_col = cls._date_to_cell(cur_dt)[0]
         new_yr = cur_dt.year+d
         new_dt = cls._cell_to_date_clamped(cls._target_col, cur_dt.month-1, new_yr)
-        GUI.cursor_set(new_dt,0)
-        GUI.today_toggle_date = None
+        cls.cursor_set_date(new_dt, reset_target_col=False)
+        cls.redraw(en_changes=False)
+        View._today_toggle_date = None
 
 
     @classmethod
     def _cursor_move_today(cls) -> None:
         # Callback for user toggling cursors between today & another date
         today = dt_date.today()
-        if GUI.cursor_date != today:
-            GUI.today_toggle_date = GUI.cursor_date
-            GUI.today_toggle_idx = GUI.cursor_idx_in_date
-            GUI.cursor_set(today,0)
-            cls._target_col = None
-        elif GUI.today_toggle_date is not None:
-            GUI.cursor_set(GUI.today_toggle_date,GUI.today_toggle_idx)
-            cls._target_col = None
+        if View._cursor_date != today:
+            View._today_toggle_date = View._cursor_date
+            View._today_toggle_idx = View._cursor_idx_in_date
+            cls.cursor_set_date(today, reset_target_col=True)
+            cls.redraw(en_changes=False)
+        elif View._today_toggle_date is not None:
+            cls.cursor_set_date(View._today_toggle_date, idx=View._today_toggle_idx, reset_target_col=True)
+            cls.redraw(en_changes=False)
 
 
     @classmethod
     def _idxcursor_move_up(cls) -> None:
         # Callback to move entry/index cursor (=cursor in bottom section) up
-        if GUI.cursor_idx_in_date>0:
-            GUI.cursor_idx_in_date -= 1
+        if View._cursor_idx_in_date>0:
+            View._cursor_idx_in_date -= 1
             cls._hide_entry_cursor()
             cls._show_entry_cursor()
 
@@ -505,8 +540,8 @@ class View_Year(View_DayUnit_Base):
     @classmethod
     def _idxcursor_move_dn(cls) -> None:
         # Callback to move entry/index cursor (=cursor in bottom section) down
-        if GUI.cursor_idx_in_date<cls._date_content_count-1:
-            GUI.cursor_idx_in_date += 1
+        if View._cursor_idx_in_date<cls._date_content_count-1:
+            View._cursor_idx_in_date += 1
             cls._hide_entry_cursor()
             cls._show_entry_cursor()
 
@@ -549,8 +584,8 @@ class View_Year(View_DayUnit_Base):
         # Callback. Called whenever events area at bottom is clicked/tapped.
         # Move entry cursor to entry that was clicked
         new_idx = cls.y_to_day_row(cls._date_content, ev.y, cls._date_content_count, cls._date_content_scroll)
-        if GUI.cursor_idx_in_date != new_idx:
-            GUI.cursor_idx_in_date = new_idx
+        if View._cursor_idx_in_date != new_idx:
+            View._cursor_idx_in_date = new_idx
             cls._hide_entry_cursor()
             cls._show_entry_cursor()
 
@@ -558,8 +593,8 @@ class View_Year(View_DayUnit_Base):
     @classmethod
     def _jump_to_date(cls, dt:dt_date) -> None:
         # Idle callback to move grid cursor to given date, e.g. on click
-        GUI.cursor_set(dt,0)
-        cls._target_col = None
+        cls.cursor_set_date(dt, reset_target_col=True)
+        cls.redraw(en_changes=False)
 
 
     @staticmethod
